@@ -166,8 +166,9 @@ def zip_file(filename,fdlist):
     zf.close()
 
 
-def unzip_file(filename, dir=None):
+def unzip_file(filename, dir=None, verbose=False):
     fname = os.path.abspath(filename)
+    logger.info("Unzipping from file '%s'" % fname)
     zf = zipfile.ZipFile(fname, 'r')
     if dir is None:
         dir = os.getcwd()
@@ -186,6 +187,7 @@ def unzip_file(filename, dir=None):
             OUTPUT.write(zf.read(name))
             OUTPUT.close()
     zf.close()
+    logger.info("Unzipping done.")
 
 
 
@@ -199,29 +201,29 @@ class Repository(object):
     svn = "svn"
     dev = []
 
-    def __init__(self, name, root=None, trunk=None, stable=None, release=None, tag=None, pyname=None, pypi=None, dev=False, username=None, install=True, rev=None, local=None, platform=None):
-        class _TEMP_(object): pass
+    def __init__(self, name, **kwds):
+        class _TEMP_(object):
+            def __init__(self, root=None, trunk=None, stable=None, release=None, tag=None, pyname=None, pypi=None, dev=False, username=None, install=True, rev=None, local=None, platform=None, version=None, branch=None):
+                import inspect
+                args, varargs, varkw, defaults = inspect.getargspec(self.__init__)
+                for i in range(len(args)-1):
+                    kwd = args[i+1]
+                    setattr(self, kwd, defaults[i])
         self.config = _TEMP_()
-        self.config.name=name
-        self.config.root=root
-        self.config.trunk=trunk
-        self.config.stable=stable
-        self.config.release=release
-        self.config.tag=tag
-        self.config.pyname=pyname
-        self.config.pypi=pypi
-        self.config.local=local
-        self.config.platform=platform
-        if dev == 'True' or dev is True:
-            self.config.dev=True
-        else:
-            self.config.dev=False
-        self.config.username=username
-        if install == 'False' or install is False:
-            self.config.install=False
-        else:
-            self.config.install=True
-        self.config.rev=rev
+        self.config.name = name
+        for kwd in kwds:
+            if kwd == 'dev':
+                if kwds[kwd] == 'True' or kwds[kwd] is True:
+                    self.config.dev = True
+                else:
+                    self.config.dev = False
+            elif kwd == 'install':
+                if kwds[kwd] == 'False' or kwds[kwd] is False:
+                    self.config.install = False
+                else:
+                    self.config.install = True
+            else:
+                setattr(self.config, kwd, kwds[kwd])
         self.initialize(self.config)
 
     def initialize(self, config):
@@ -229,6 +231,7 @@ class Repository(object):
         self.root = config.root
         self.trunk = None
         self.trunk_root = None
+        self.branch = None
         self.stable = None
         self.stable_root = None
         self.release = None
@@ -242,6 +245,7 @@ class Repository(object):
             self.platform_re = None
         else:
             self.platform_re = re.compile(config.platform)
+        self.version = config.version
         if not config.pypi is None:
             self.pyname=config.pypi
         else:
@@ -273,7 +277,10 @@ class Repository(object):
                         rootdir_output = urllib2.urlopen(self.config.root).read()
                     else:
                         rootdir_output = urllib2.urlopen(self.config.root, timeout=30).read()
-            self.trunk = self.config.root+'/trunk'
+            if self.config.branch:
+                self.trunk = self.config.root+'/branches/'+self.config.branch
+            else:
+                self.trunk = self.config.root+'/trunk'
             self.trunk_root = self.trunk
             try:
                 if offline or not 'stable' in rootdir_output:
@@ -341,6 +348,8 @@ class Repository(object):
         elif not config.pyname is None:
             print >>OUTPUT, 'pypi=%s' % config.pyname
         print >>OUTPUT, 'dev=%s' % str(config.dev)
+        if not config.branch is None:
+            print >>OUTPUT, 'branch=%s' % str(config.branch)
         print >>OUTPUT, 'install=%s' % str(config.install)
         if not config.rev is None:
             print >>OUTPUT, 'rev=%s' % str(config.rev)
@@ -348,6 +357,8 @@ class Repository(object):
             print >>OUTPUT, 'username=%s' % str(config.username)
         if not config.platform is None:
             print >>OUTPUT, 'platform=%s' % config.platform
+        if not config.version is None:
+            print >>OUTPUT, 'version=%s' % config.version
 
 
     def find_pkgroot(self, trunk=False, stable=False, release=False):
@@ -399,7 +410,7 @@ class Repository(object):
                 self.pkgroot = self.release_root
 
         else:
-            raise IOError, "Must have one of trunk, stable or release specified"
+            raise IOError, "Must have one of trunk, stable or release specified: %s" % self.name
 
 
     def install_trunk(self, dir=None, install=True, preinstall=False, offline=False):
@@ -417,6 +428,8 @@ class Repository(object):
     def perform_install(self, dir=None, install=True, preinstall=False, offline=False):
         if not self.platform_re is None and not self.platform_re.match(sys.platform):
             return
+        if not self.version is None and not eval(self.version):
+            return
         if self.pkgdir is None and self.local is None:
             self.easy_install(install, preinstall, dir, offline)
             return
@@ -603,6 +616,11 @@ class Installer(object):
         if not 'root' in kwds and not 'pypi' in kwds and not 'release' in kwds and not 'trunk' in kwds and not 'stable' in kwds:
             raise IOError, "No repository info specified for repository "+args[0]
         repos = Repository( *args, **kwds)
+        if repos.name in self.sw_dict:
+            for i in range(len(self.sw_packages)):
+                if self.sw_packages[i].name == repos.name:
+                    self.sw_packages.pop(i)
+                    break
         self.sw_dict[repos.name] = repos
         self.sw_packages.append( repos )
 
@@ -803,29 +821,6 @@ class Installer(object):
         if not self.config_file is None and not (options.trunk or options.stable or options.release):
             self.config_file = os.path.dirname(self.config_file)+"/pypi.ini"
         #
-        # Parse config files
-        #
-        if options.update:
-            self.config=None
-            options.config_files.append( join(self.abshome_dir, 'admin', 'config.ini') )
-        if not self.config is None and (len(options.config_files) == 0 or options.keep_config):
-            fp = StringIO.StringIO(self.config)
-            self.read_config_file(fp=fp, follow_externals=options.follow_externals)
-            fp.close()
-        if not self.config_file is None and (len(options.config_files) == 0 or options.keep_config):
-            self.read_config_file(file=self.config_file, follow_externals=options.follow_externals)
-        for file in options.config_files:
-            self.read_config_file(file=file, follow_externals=options.follow_externals)
-        print "-----------------------------------------------------------------"
-        print "Finished processing configuration information."
-        print "-----------------------------------------------------------------"
-        print " START - Configuration summary"
-        print "-----------------------------------------------------------------"
-        self.write_config(stream=sys.stdout)
-        print "-----------------------------------------------------------------"
-        print " END - Configuration summary"
-        print "-----------------------------------------------------------------"
-        #
         # If applying preinstall, then only do subversion exports
         #
         if options.preinstall:
@@ -922,7 +917,31 @@ class Installer(object):
         #
         for file in options.zip:
             unzip_file(file, dir=self.abshome_dir)
-
+        #
+        # Parse config files
+        #
+        if options.update or options.release:
+            if os.path.exists(join(self.abshome_dir, 'admin', 'config.ini')):
+                self.config=None
+                options.config_files.append( join(self.abshome_dir, 'admin', 'config.ini') )
+        if not self.config is None and (len(options.config_files) == 0 or options.keep_config):
+            fp = StringIO.StringIO(self.config)
+            self.read_config_file(fp=fp, follow_externals=options.follow_externals)
+            fp.close()
+        if not self.config_file is None and (len(options.config_files) == 0 or options.keep_config):
+            self.read_config_file(file=self.config_file, follow_externals=options.follow_externals)
+        for file in options.config_files:
+            self.read_config_file(file=file, follow_externals=options.follow_externals)
+        print "-----------------------------------------------------------------"
+        print "Finished processing configuration information."
+        print "-----------------------------------------------------------------"
+        print " START - Configuration summary"
+        print "-----------------------------------------------------------------"
+        self.write_config(stream=sys.stdout)
+        print "-----------------------------------------------------------------"
+        print " END - Configuration summary"
+        print "-----------------------------------------------------------------"
+        #
         if options.preinstall or not options.offline:
             #self.get_packages(options)
             pass
@@ -1019,6 +1038,10 @@ class Installer(object):
         print >>OUTPUT, self.README.strip()
         OUTPUT.close()
         #
+        # Finalize package export
+        #
+        self.finalize_packages(options)
+        #
         # Finalize preinstall
         #
         if options.preinstall:
@@ -1036,6 +1059,12 @@ class Installer(object):
         #
         pass
 
+    def finalize_packages(self, options):
+        #
+        # Perform final steps need to get packages
+        #
+        pass
+
     def install_packages(self, options):
         #
         # Set the bin directory
@@ -1046,8 +1075,6 @@ class Installer(object):
             bindir = join(self.abshome_dir,"bin")
         if is_jython:
             Repository.python = os.path.abspath(join(bindir, 'jython.bat'))
-        #elif sys.platform.startswith('win'):
-            #Repository.python = os.path.abspath(join(bindir, 'python'))
         else:
             Repository.python = os.path.abspath(join(bindir, 'python'))
         if os.path.exists(os.path.abspath(join(bindir, 'easy_install'))):
